@@ -17,12 +17,14 @@
 
 import logging
 import os
+import re
 import sys
 import tempfile
 import shutil
 import importlib
 import tarfile
 import inspect
+
 try:
     import cPickle as pickle
 except ImportError:
@@ -56,14 +58,13 @@ class SparkSubmitTask(ExternalProgramTask):
     # Spark applications write its logs into stderr
     stream_for_searching_tracking_url = 'stderr'
 
-    def run(self):
+    @property
+    def tracking_url_pattern(self):
         if self.deploy_mode == "cluster":
             # in cluster mode client only receives application status once a period of time
-            self.tracking_url_pattern = r"tracking URL: (https?://.*)\s"
+            return r"tracking URL: (https?://.*)\s"
         else:
-            self.tracking_url_pattern = r"Bound (?:.*) to (?:.*), and started at (https?://.*)\s"
-
-        super(SparkSubmitTask, self).run()
+            return r"Bound (?:.*) to (?:.*), and started at (https?://.*)\s"
 
     def app_options(self):
         """
@@ -298,8 +299,9 @@ class PySparkTask(SparkSubmitTask):
         return [self.app, pickle_loc] + self.app_options()
 
     def run(self):
-        self.run_path = tempfile.mkdtemp(prefix=self.name)
-        self.run_pickle = os.path.join(self.run_path, '.'.join([self.name.replace(' ', '_'), 'pickle']))
+        path_name_fragment = re.sub(r'[^\w]', '_', self.name)
+        self.run_path = tempfile.mkdtemp(prefix=path_name_fragment)
+        self.run_pickle = os.path.join(self.run_path, '.'.join([path_name_fragment, 'pickle']))
         with open(self.run_pickle, 'wb') as fd:
             # Copy module file to run path.
             module_path = os.path.abspath(inspect.getfile(self.__class__))
